//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Modul:       - Dieses Modul stellt Funktionen fr die direkte Kommunikation
//                mit dem USB-Treiber bereit
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#include <windows.h>
#include <stdio.h>
#include "usb.h"


HANDLE g_hUSB = NULL;
             
#define IOCTL_INDEX         (0x0000)
#define IOCTL_BULK_READ     (0x220002 + (IOCTL_INDEX + 19) * 4)
#define IOCTL_BULK_WRITE    (0x220001 + (IOCTL_INDEX + 20) * 4)

typedef struct 
{
  unsigned char start_id;
  unsigned int  len;
  unsigned long address;
  unsigned int  record_type;
  unsigned char *data;
  int           checksum;
  unsigned char cr;
  unsigned char lf;

} t_hex_data;



//*****************************************************************************
// Funktion:    Initialisiert den Treiber u. stellt Verbindung zur USB-Pipe her
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      gibt zurck, ob die Initialisierung erfolgreich war
//-----------------------------------------------------------------------------
// Datum:       2005-06-05, Andreas Schibilla
//*****************************************************************************
int usb_init(void)
{
 char str[50];
 sprintf(str, "\\\\.\\atmusb-%d", 0);

 g_hUSB = CreateFile(str, GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);

 return g_hUSB != INVALID_HANDLE_VALUE;
}



//*****************************************************************************
// Funktion:    Beendet die Verbindung zur USB Pipe mit dem Treiber
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-05, Andreas Schibilla
//*****************************************************************************
void usb_free(void)
{
 if (g_hUSB != INVALID_HANDLE_VALUE) 
     CloseHandle(g_hUSB);
}



//*****************************************************************************
// Funktion:    Sendet Daten zum BULK-Port des Gerts
//-----------------------------------------------------------------------------
// Parameter:   pn      - PipeNum: Welche Pipe am HUB wird verwendet 
//                        (Default=0)
//              buf     - Zeiger auf Buffer, der die Daten enthlt
//              anzahl  - Anzahl Bytes, die gesendet werden sollen
// Return:      gibt zurck, ob das Senden erfolgreich war
//-----------------------------------------------------------------------------
// Datum:       2005-06-05, Andreas Schibilla
//*****************************************************************************
int usb_send_bulk(unsigned char pn, char * buf, int anzahl)
{
 unsigned long nBytes;
 unsigned long PipeNum = pn;

 return DeviceIoControl(g_hUSB, IOCTL_BULK_WRITE, &PipeNum, sizeof(PipeNum), buf, anzahl, &nBytes, NULL);
}



//*****************************************************************************
// Funktion:    Empfngt Daten vom BULK-Port des Gerts
//-----------------------------------------------------------------------------
// Parameter:   pn      - PipeNum: Welche Pipe am HUB wird verwendet (Default=0)
//              buf     - Zeiger auf Buffer, in den die Daten gelesen werden
//                        sollen
//              anzahl  - max. Gre des Buffers (buf)
// Return:      gibt zurck, wie viele Bytes gelesen wurden
//              oder -1, falls Lesen nicht erfolgreich war!
//-----------------------------------------------------------------------------
// Datum:       2005-06-05, Andreas Schibilla
//*****************************************************************************
int usb_get_bulk(unsigned char pn, char *buf, int anzahl)
{
 unsigned long nBytes;
 unsigned long PipeNum = pn;

 int bRc = DeviceIoControl(g_hUSB, IOCTL_BULK_READ, &PipeNum, sizeof(PipeNum), buf, anzahl, &nBytes, NULL);
 return bRc ? nBytes : -1;
}

