//*****************************************************************************
//                    Laborprojekt, FH-Wedel 2005
//-----------------------------------------------------------------------------
//
// Modul:       - enthlt Funktionen zum Abschicken und Empfangen 
//                der Programmerbefehle
//
//-----------------------------------------------------------------------------
// Datum:       Copyright 2005, Andreas Schibilla
//*****************************************************************************
#include <windows.h>
#include <shlwapi.h>
#include <stdio.h>
#include <math.h>
#include "usb.h"
#include "log.h"
#include "commands.h"



//*****************************************************************************
// Funktion:    berprft eine Binrdatei und gibt zurck, ob sie gltig
//              (d.h. nicht zu gro und lesbar) ist. Auerdem wird die 
//              Dateigre angezeigt.
//-----------------------------------------------------------------------------
// Parameter:   filename  - Name der Datei, die getestet wird
//              dwSize    - bekommt die Dateigre
// Return:      gibt zurck, ob die Binrdatei gtlig ist
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL isBinFile(char *filename, DWORD *dwSize)
{
 HANDLE hFile;
 
 // Kann Datei zum Lesen geffnet werden?
 hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
 if (hFile == INVALID_HANDLE_VALUE)
    return FALSE;

 *dwSize = GetFileSize(hFile, NULL);
 CloseHandle(hFile);

 return ( (*dwSize <= C_MAX_FLASH_SIZE) && (*dwSize > 0) );
}



//*****************************************************************************
// Funktion:    berprft eine Hexdatei und gibt zurck, ob sie gltig
//              (d.h. lesbar und Dateiendung .HEX oder .IHX) ist. 
//-----------------------------------------------------------------------------
// Parameter:   filename  - Name der Datei, die getestet wird
// Return:      gibt zurck, ob die Hexdatei gtlig ist
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL isHexFile(char *filename)
{
 // Kann Datei zum Lesen geffnet werden?
 HANDLE hFile = CreateFile(filename, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
 if (hFile == INVALID_HANDLE_VALUE)
    return FALSE;

 CloseHandle(hFile);

 // Hat Datei die Endung .HEX oder .IHX?
 char * p = PathFindExtension(filename);
 
 if (!_stricmp(p, ".hex") )
    return TRUE;
 
 if (!_stricmp(p, ".ihx") )
    return TRUE;
   
 return FALSE;
}



//*****************************************************************************
// Funktion:    Wandelt einen einstelligen Ascii-Hex-Wert in eine 
//              Dezimalzahl um
//-----------------------------------------------------------------------------
// Parameter:   n   -  Hex-Wert (in Ascii)
//              res -  Ergebnis als Dezimalzahl
// Return:      gibt zurck, ob das Umwandeln geklappt hat (ob Rckgabewert in
//              res gltig ist)
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
BOOL GetHex(unsigned char n, unsigned char *res)
{
 *res = 0;
 if ( (n >= 'a') && (n <= 'f') )       // Kleinbuchstaben von a-f bearbeiten
 {
    *res = n - 'a' +10;
    return TRUE;
 }
 if ( (n >= 'A') && (n <= 'F') )       // Grobuchstaben von A-F bearbeiten
 {
    *res = n - 'A' +10;
    return TRUE;
 }
 if ( (n >= '0') && (n <= '9') )       // Zahlen von 0-9 bearbeiten
 {
    *res = n - '0';
    return TRUE;
 }

 return FALSE;
}



//*****************************************************************************
// Funktion:    Wandelt ein Byte von der Ascii-Hex-Darstellung in einen
//              "normalen" Bytewert um
//-----------------------------------------------------------------------------
// Parameter:   d1  - 1. Hex-Wert (Low Byte in Ascii)
//              d0  - 2. Hex-Wert (High Byte in Ascii)
//              res - Ergebnisbyte
// Return:      gibt zurck, ob das Umwandeln geklappt hat (ob Rckgabewert in
//              res gltig ist)
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
BOOL GetAscii(unsigned char d1, unsigned char d0, unsigned char *res)
{
 unsigned char a,b;

 if (GetHex(d0, &a) && GetHex(d1, &b) )
 {
    *res = b*16+a;                  // Low-Byte ist um 4 Bits verschoben (2^4=16)
    return TRUE;
 }

 return FALSE;
}



//*****************************************************************************
// Funktion:    liest aus einer Zeile einer Hexdatei das Hex-Record in die
//              entspr. Struktur "t_hex_record" ein
//-----------------------------------------------------------------------------
// Parameter:   hr    - Ziel, nimmt das Hex-Record auf
//              p     - Zeiger auf den Anfang der Zeile (String)
//              len   - Lnge der Zeile
// Return:      gibt zurck, ob das Auslesen geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
BOOL ReadHexRecord(t_hex_record *hr, char *p, size_t len)
{
 unsigned char a,b;
 unsigned char checksum = 0;

 if (len < 11)                          // string p zu kurz, nichtmal Header des Records passt da rein
    return FALSE;

 if (p[0] != ':')                       // das erste Zeichen muss ein ':' sein
    return FALSE;

 if (!GetAscii(p[1], p[2], &a) )        // Versuche die Lnge der Datenbytes zu ermitteln
    return FALSE;
 checksum += a;
 hr->len = a;

 if (!GetAscii(p[3], p[4], &a) )        // Low-Anteil der Adresse in a speichern
    return FALSE;
 checksum += a;

 if (!GetAscii(p[5], p[6], &b) )        // High-Anteil der Adresse in b speichern
    return FALSE;
 checksum += b;

 hr->addr = (a << 8);                   // Adresse zusammenbauen
 hr->addr += b;
 
 if (!GetAscii(p[7], p[8], &a) )        // Versuche Record-Typ zu ermitteln
    return FALSE;
 checksum += a;
 hr->type = a;

 if ( (size_t)(2*hr->len+11) > len)     // Record enthlt nicht alle Datenbytes?
    return FALSE;

 int pos=9;                             
 for (int i=0; i < (hr->len); ++i)            // Lese die Hex-Datenbytes ein
 {

    if (!GetAscii(p[pos], p[pos+1], &a) )     // ein Datenbyte einlesen
        return FALSE;

    hr->data[i] = a;                          // und abspeichern
    checksum += a;
    pos += 2;
 }
 
 if (!GetAscii(p[pos], p[pos+1], &a) )        // Checksumme auslesen
    return FALSE;

 checksum = 0 - checksum;                     // berechnete Checksumme mit der
 if ( checksum != a)                          // aus dem Hex-Record vergleichen
    return FALSE;


 return TRUE;
}



//*****************************************************************************
// Funktion:    liest eine Hexdatei in den Speicher ein und markiert 
//              alle zu schreibenden Bytes
//-----------------------------------------------------------------------------
// Parameter:   hexfile   - Name der Hexdatei
//              m         - Speicherstruktur, die das Ergebnis aufnimmt
//              flashsize - Gre des Flashspeichers
// Return:      gibt zurck, ob das Einlesen geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL read_hexfile(char *hexfile, t_mem *m, int flashsize)
{
 FILE               *stream;
 char               line[1025];
 t_hex_record       hr;
 size_t             num       = 0; 
 unsigned int       last_addr = 0;
 unsigned int       cnt       = 1;

 
 ZeroMemory (m, sizeof(t_mem) );
 if( (stream = fopen(hexfile, "r" )) != NULL )
 {
      while( fgets(line, 1025, stream ) != NULL)
      {
            size_t len = strlen(line);
            if (!ReadHexRecord(&hr, line, len) )
            {
                Log("\r\nERR: Ungltige Hex-Datei! Fehler in Zeile %d\r\n", cnt);
                fclose( stream );
                return FALSE;
            }
            num += hr.len;
            if ( (hr.addr+hr.len) > flashsize )
            {
                Log("\r\nERR: Adresse auerhalb des Flash-Speichers, Zeile wird ausgelassen\r\n");
                ++cnt;
                continue;   
            }
            
            // Hex-Record in Speicher-Abbild ablegen
            int offset = hr.addr;
            for (int i=0; i<hr.len; ++i)
            {
                m->data[offset+i]      = hr.data[i];
                m->enable[offset+i]    = TRUE;
            }

            ++cnt;

      }
      fclose( stream );
 }
 else
 {
    Log("\r\nERR: Konnte Datei '%s' nicht zum Lesen ffnen!\r\n", hexfile);
    return FALSE;
 }

 return TRUE;
}



//*****************************************************************************
// Funktion:    liest eine Binrdatei in den Speicher ein und markiert 
//              alle zu schreibenden Bytes
//-----------------------------------------------------------------------------
// Parameter:   binfile   - Name der Datei
//              m         - Speicherstruktur, die das Ergebnis aufnimmt
//              flashsize - Gre des Flashspeichers
// Return:      gibt zurck, ob das Einlesen geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL read_binfile(char *binfile, t_mem *m, unsigned int flashsize)
{
 HANDLE   hFile;
 DWORD    dwSize, nBytesRead;
 BOOL     bResult;

 ZeroMemory (m,sizeof(t_mem) ); 

 // Kann Datei zum Lesen geffnet werden?
 hFile = CreateFile(binfile, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, 0, NULL);
 if (hFile == INVALID_HANDLE_VALUE)
 {
    Log("\r\nERR: Konnte die Binrdatei '%s' nicht zum Lesen ffnen!", binfile);
    return FALSE;
 }                                 
 
 // Ist Datei zu gro? 
 dwSize = GetFileSize(hFile, NULL);
 if (dwSize > flashsize)
 {
    Log("\r\nERR: Die Binrdatei '%s' ist zu gro!", binfile);
    CloseHandle(hFile);
    return FALSE;
 }
 
 // Lese die gesamte Datei in den Speicher
 bResult = ReadFile(hFile, m->data, dwSize, &nBytesRead, NULL);
 if(!bResult || (nBytesRead != dwSize) )
 {
    Log("\r\nERR: Konnte die Binrdatei '%s' nicht in den Speicher einlesen!", binfile);
    CloseHandle(hFile);
    return FALSE;
 }
 
 // Alle Bytes des Speicherabbilds werden als "Enable" (zum Flashen) markiert
 for (DWORD i=0; i<dwSize; ++i)
    m->enable[i] = TRUE;

 return TRUE;
}



//*****************************************************************************
// Funktion:    Zeigt ein USB-Paket in Hex-Form an
//-----------------------------------------------------------------------------
// Parameter:   data  - Datenbytes des Pakets, die angezeigt werden sollen
//              len   - Anzahl Bytes
// Return:      -
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
void show_packet(char *data, size_t len)
{
 Log("HEX: ");
 for(size_t i=0; i<len; ++i)
 {
   char buf[20];
   _ltoa((unsigned char) data[i], buf, 16);
   Log(buf);
   Log(" ");
 }
 Log("\r\n");
}



//*****************************************************************************
// Funktion:    Sendet ein Paket ber USB zum Programmer und empfngt ein
//              Antwortpaket
//-----------------------------------------------------------------------------
// Parameter:   d_in  - Paket, das verschickt werden soll
//              d_out - Antwort Paket
// Return:      gibt zurck, ob das Senden geklappt hat und eine positive
//              Besttigung angekommen ist
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL send_packet(t_packet *d_in, t_packet *d_out)
{
 
 //--- Treiber initialiseren
 if (!usb_init() )
 {
  Log("\r\nERR: Konnte den USB-Treiber nicht initialisieren!\r\n");
  return FALSE;
 }
 
 //--- out-Packet berprfen
 if (d_out->sig != PACKET_SIG)
 {
  Log("\r\nERR: Kann keinen Befehl mit ungltiger Signatur verschicken!\r\n");
  usb_free();
  return FALSE;
 }
 
 //--- max. Paketgre eingehalten?
 if ( d_out->len > (PACKET_SIZE-2) )
 {
  Log("\r\nERR: Maximale Paketgre berschritten, Befehl kann nicht gesendet werden!\r\n");
  usb_free();
  return FALSE;
 }

 //--- Paket per USB an den Programmer senden
 if (usb_send_bulk(0, (char *) d_out, 2+d_out->len))
 {
   // Log("Gesendet (PC -> DEVICE): %d Bytes\r\n", 2+d_out->len);
   // show_packet((char *) d_out, 2+d_out->len);
 }
 else
 {
    Log("\r\nERR: Konnte Daten nicht per USB (BULK) verschicken!\r\n");
    usb_free();
    return FALSE;
 } 

 //--- Besttigung vom Gert empfangen
 ZeroMemory (d_in, sizeof(t_packet) );
 d_in->len = usb_get_bulk(1, (char *) d_in, PACKET_SIZE);
 if (d_in->len == -1)
 {
    Log("\r\nERR: Keine Daten vom Programmer empfangen!\r\n");  
    usb_free();
    return FALSE;
 }
 
 // Log("Empfangen (PC <- DEVICE): %d Bytes\r\n", d_in->len);
 // show_packet((char *) d_in, d_in->len);
 
 //--- Paket zu kurz oder falsche Signatur? 
 if ( (d_in->len < 1) || (d_in->sig != PACKET_SIG) )        
 { 
    Log("Empfangen (PC <- DEVICE): %d Bytes\r\n", d_in->len);
    show_packet((char *) d_in, d_in->len);
    Log("\r\nERR: Negative oder ungltige Besttigung empfangen!\r\n"); 
    usb_free();
    return FALSE;
 }
 
 //--- Fehler als Besttigung empfangen, gebe ihn aus
 if (d_in->command == CMD_ERROR_STR)
 { 
    char buf[64];
    ZeroMemory (buf, sizeof(buf));
    strncpy(buf, (char *) d_in->data, d_in->len);
    Log("\r\nERR: %s\r\n", buf);  
    usb_free();
    return FALSE;
 }

 usb_free();
 return TRUE; 
}



//*****************************************************************************
// Funktion:    Ermittelt den eingelten Chip-Typ und die Version 
//              des USB-Programmers
//-----------------------------------------------------------------------------
// Parameter:   sig  - speichert die ermittelten Signatur- und Versions-
//                     informationen ab
// Return:      gibt zurck, ob das Auslesen geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL chip_type_and_version(t_signature *sig)
{
 t_packet   ver_in, sig_in, d_out;

 Log("--- Ermittle Chip-Typ und Version des USB-Programmers...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig    = PACKET_SIG;
 d_out.command  = CMD_GET_VERSION;
 d_out.len    = 0;
 if (!send_packet(&ver_in, &d_out) )
 {
    Log("\r\nERR: Konnte Programmer-Version nicht ermitteln!\r\n");
    return FALSE;
 }

 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig    = PACKET_SIG;
 d_out.command  = CMD_GET_TYPE;
 d_out.len    = 0;
 if (!send_packet(&sig_in, &d_out) )
 {
    Log("\r\nERR: Konnte Chip-Typ nicht ermitteln!\r\n");
    return FALSE;
 }

 //--- Signatur ermitteln und Speichergre setzen 
 ZeroMemory (sig, sizeof(t_signature) );
 sig->mfr = sig_in.data[0];
 sig->art = sig_in.data[1];
 switch(sig->art)
 {
  case 0xff:  Log("\r\nERR: Konnte Chip-Typ nicht ermitteln!\r\n");
              return FALSE;
  case 0x21:  sig->flashsize = 2048; 
              break;
  case 0x41:  sig->flashsize = 4096; 
              break;
  case 0x11:  
  case 0x12:  
  default:    sig->flashsize = 1024;
              break;
 }

 //--- Hersteller ermitteln und Anzeigen
 char tmp[20];
 strcpy(sig->mfr_str, "[0x");
 _ltoa ((unsigned char) sig->mfr, tmp, 16);
 strcat(sig->mfr_str, tmp);
 strcat(sig->mfr_str, "] ");
 if (sig->mfr == C_SIG_ATMEL_ID)
    strcat(sig->mfr_str, C_SIG_ATMEL);
 else
    strcat(sig->mfr_str, C_SIG_UNKNOWN);
 
 //--- Produkt ermitteln und Anzeigen
 strcpy(sig->art_str, "[0x");
 _ltoa ((unsigned char) sig->art, tmp, 16);
 strcat(sig->art_str, tmp);
 strcat(sig->art_str, "] ");
 switch(sig->art)
 {
  case 0x11:  strcat(sig->art_str, C_SIG_1051);
              break;
  case 0x12:  strcat(sig->art_str, C_SIG_1051U);
              break;
  case 0x21:  strcat(sig->art_str, C_SIG_2051);
              break;
  case 0x41:  strcat(sig->art_str, C_SIG_4051);
              break;
  default:    strcat(sig->art_str, C_SIG_UNKNOWN);
              break;
 }

 strncpy(sig->version_str, (char *) ver_in.data, ver_in.len);
 sig->version_str[ver_in.len] = '\0';
 
 return TRUE;
}



//*****************************************************************************
// Funktion:    Lscht den Flashspeicher des Chips vollstndig
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      gibt zurck, ob das Lschen geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
BOOL chip_erase(void)
{
 t_packet   d_in, d_out;

 Log("--- Sende Befehl zum Lschen des Chips...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = CMD_ERASE;
 d_out.len      = 0;
 if (!send_packet(&d_in, &d_out) )
 {
    Log("\r\nERR: Konnte Chip nicht lschen!\r\n");
    return FALSE;
 }

 return TRUE;
}

   
  
//*****************************************************************************
// Funktion:    Fhrt einen Blank-Check nach dem Lschen durch
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      gibt zurck, ob der Blank-Check erfolgreich war
//-----------------------------------------------------------------------------
// Datum:       2005-06-25, Andreas Schibilla
//*****************************************************************************
BOOL chip_blank_check(void)
{
 t_packet   d_in, d_out;

 Log("--- Sende Befehl fr Blank-Check des Chips...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = CMD_BLANK_CHECK;
 d_out.len      = 0;
 if (!send_packet(&d_in, &d_out) )
 {
    Log("\r\nERR: Blank-Check nicht erfolgreich!\r\n");
    return FALSE;
 }

 return TRUE;
}



//*****************************************************************************
// Funktion:    Debug-Funktion, liest 64 Bytes aus dem Chip aus
//-----------------------------------------------------------------------------
// Parameter:   -
// Return:      gibt zurck, ob die 64 Bytes empfangen wurden
//-----------------------------------------------------------------------------
// Datum:       2005-06-26, Andreas Schibilla
//*****************************************************************************
BOOL chip_debug(void)
{
 t_packet   d_in, d_out;

 Log("--- Bitte 64 bytes des Chips schicken...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = CMD_DEBUG;
 d_out.len      = 0;
 if (!send_packet(&d_in, &d_out) )
 {
    Log("Debug-Fkt!!!\r\n");
    return FALSE;
 }
 show_packet((char *) &d_in, d_in.len);
 return TRUE;
}



//*****************************************************************************
// Funktion:    Sendet Hexdaten an den Programmer um den Chip zu Programmieren
//              oder zur Verifizierung
//-----------------------------------------------------------------------------
// Parameter:   hexfile   - Dateiname mit den Flashdaten
//              isHex     - gibt an, ob die Datei mit den Daten eine Intel-
//                          HEX-Datei (TRUE) oder eine Binrdatei ist (FALSE)
//              command   - Befehl: CMD_PROGRAM oder CMD_VERIFY
// Return:      gibt zurck, ob das Programmieren/Verifizieren geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-28, Andreas Schibilla
//*****************************************************************************
BOOL chip_program(char *hexfile, BOOL isHex, unsigned char command)
{
 t_packet           d_in, d_out;
 t_mem              m;
 size_t             num       = 0; 
 unsigned int       last_addr = 0;
 unsigned int       cnt       = 1;
 
 //--- Speicherabbild aus Datei erstellen
 if (isHex)
 {
        // Speicherabbild aus Hex-Datei erstellen
        if (!read_hexfile(hexfile, &m, C_MAX_FLASH_SIZE) )
        {
            Log("\r\nERR: Konnte kein Speicherabbild der Hex-Datei '%s' erstellen!\r\n", hexfile);
            return FALSE;
        }
 }
 else
 {
        // Speicherabbild aus Binrdatei erstellen
        if (!read_binfile(hexfile, &m, C_MAX_FLASH_SIZE) )
        {
            Log("\r\nERR: Konnte kein Speicherabbild der Binrdatei '%s' erstellen!\r\n", hexfile);
            return FALSE;
        }
 }

 //--- Zu Beginn: Programmier/Verify-Anfrage schicken
 Log("--- Sende neues Programm (Hexfile) zum Gert...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = command;
 d_out.len      = 0;
 if (!send_packet(&d_in, &d_out) )
 {
    Log("\r\nERR: Konnte keine Hexfile-Anfrage an den Programmer schicken!\r\n");
    return FALSE;
 }

 BOOL bSendPacket = FALSE;
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = CMD_HEXDATA;
 d_out.len      = 4;                // noch keine Datenbytes enthalten
 cnt            = 0;
 int datacount  = 0;

 //--- die Hex-Daten schicken
 for(unsigned short i=0; i<C_MAX_FLASH_SIZE; ++i)       // durchlaufe alle Bytes des Speicherabbilds
 {

    if (m.enable[i])
    {
            // AddToPacket
            if (d_out.len == 4)                         // Paket noch leer?
            {
                d_out.data[0] = 0x00;                   // Lnge
                d_out.data[1] = HIBYTE(i);              // Adresse
                d_out.data[2] = LOBYTE(i);
                d_out.data[3] = 0x00;                   // Typ
            }
            d_out.data[4+d_out.data[0]] = m.data[i];    // Datenbyte hinzufgen
            d_out.data[0]++;                            // Lnge inkrementieren
            d_out.len = 4+d_out.data[0];                // Anzahl Bytes in d_out.data[]


            // Fall 1: Ok, danach ist noch mehr platz
            // Fall 2: Ok, danach ist kein Platz mehr oder Letztes Byte -> Paket abschicken und anschl. clearen
            if ( (d_out.len == (PACKET_SIZE-2)) || (i==C_MAX_FLASH_SIZE-1) )
                bSendPacket = TRUE;
    }
    else
    {
            // Fall 1: Paket ist leer -> nix tun, weiter zur nchsten Adresse
            // Fall 2: Paket enthlt schon Bytes -> Paket absenden und anschl. clearen
            if (d_out.len > 4)
                bSendPacket = TRUE;
    }


    // Paket abschicken
    if (bSendPacket)
    {
        ++cnt;
        datacount       += d_out.data[0];
        if (!send_packet(&d_in, &d_out) )
        {
          Log("\r\nERR: Fehler beim Senden von Paket %d!\r\n", cnt);
          return FALSE;
        }
        Log("#");   
        // und anschl. clearen       
        ZeroMemory (&d_out, sizeof(d_out) );
        d_out.sig       = PACKET_SIG;
        d_out.command   = CMD_HEXDATA;
        d_out.len       = 4;                // noch keine Datenbytes enthalten
        bSendPacket     = FALSE; 
    }

 }
 
 //--- Endepaket abschicken
 Log("\r\n--- Sende Letztes Paket, alles bertragen...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = CMD_HEXDATA;
 d_out.len      = 0;
 d_out.data[0]  = 0x00; // Lnge
 d_out.data[1]  = 0x00; // Adresse
 d_out.data[2]  = 0x00;
 d_out.data[3]  = 0x01; // Typ
 d_out.len = 4;
 if (!send_packet(&d_in, &d_out) )
 {
    Log("\r\nERR: Konnte letztes Hexfile-Paket nicht senden!\r\n");
    return FALSE;
 }
 
 Log("\r\n%d Bytes bertragen!\r\n", datacount);
 return TRUE;
}



//*****************************************************************************
// Funktion:    Setz die Lockbits des Chips
//-----------------------------------------------------------------------------
// Parameter:   lb - Welche Lockbits sollen geschrieben werden?
//                   01 = ein Lockbit
//                   02 = beide lockbits
// Return:      gibt zurck, ob das Setzen der Lockbits geklappt hat
//-----------------------------------------------------------------------------
// Datum:       2005-06-29, Andreas Schibilla
//*****************************************************************************
BOOL chip_set_lockbits(unsigned char lb)
{
 t_packet   d_in, d_out;

 Log("--- Sende Befehl zum Setzen der Lockbits...\r\n");
 ZeroMemory (&d_out, sizeof(d_out) );
 d_out.sig      = PACKET_SIG;
 d_out.command  = CMD_SET_LOCK;
 d_out.data[0]  = lb;
 d_out.len      = 1;
 if (!send_packet(&d_in, &d_out) )
 {
    Log("\r\nERR: Lockbits konnten nicht gesetzt werden!\r\n");
    return FALSE;
 }

 return TRUE;
}


